/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.bu.loader.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tinygroup.bu.BusinessUnitManager;
import org.tinygroup.bu.BusinessUnitManagerFactory;
import org.tinygroup.bu.config.BusinessUnitInfo;
import org.tinygroup.bu.config.BusinessUnitJarDependency;
import org.tinygroup.bu.config.BusinessUnitJarExtend;
import org.tinygroup.bu.loader.BusinessUnitLoader;
import org.tinygroup.commons.classloader.ResourceURLClassLoader;
import org.tinygroup.loader.LoaderManagerFactory;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.pluginresource.PluginResourceFactory;
import org.tinygroup.vfs.FileObject;
import org.tinygroup.vfs.VFS;
import org.tinygroup.xstream.XStreamFactory;

import com.thoughtworks.xstream.XStream;

public class BusinessUnitLoaderImpl implements BusinessUnitLoader {
	private static Logger logger = LoggerFactory
			.getLogger(BusinessUnitLoaderImpl.class);
	/**
	 * BU的包，及 其依赖的jar包url
	 */
	private Map<FileObject, List<URL>> fileDependJar = new HashMap<FileObject, List<URL>>();
	/**
	 * BU的包，及 其依赖的jar包url
	 */
	private Map<FileObject, URL> fileUrl = new HashMap<FileObject, URL>();

	/**
	 * BU的包，及其外部jar包url
	 */
	private Map<FileObject, List<URL>> fileExtendJar = new HashMap<FileObject, List<URL>>();
	/**
	 * bu包业务单元及对应的loader
	 */
	private Map<FileObject, ClassLoader> buLoaders = new HashMap<FileObject, ClassLoader>();
	/**
	 * 公共包对应的loader
	 */
	private ResourceURLClassLoader publicLoader = new ResourceURLClassLoader(
			new URL[0], this.getClass().getClassLoader());;

	private BuFileResolver resoler = new BuFileResolver();

	public List<BusinessUnitInfo> listJar(String path) {
		List<BusinessUnitInfo> list = new ArrayList<BusinessUnitInfo>();
		load(path, list, false);
		return list;
	}

	public List<BusinessUnitInfo> load(String path) {
		List<BusinessUnitInfo> list = new ArrayList<BusinessUnitInfo>();
		load(path, list, true);
		return list;
	}

	public void load(String path, List<BusinessUnitInfo> list, boolean isLoad) {
		logger.logMessage(LogLevel.DEBUG, "业务单元扫描:传入的路径为:{0}", path);

		if (path == null || "".equals(path)) {
			logger.logMessage(LogLevel.DEBUG, "传入的文件名为空");
		}
		FileObject fileD = null;
		fileD = VFS.resolveFile(path);
		if (fileD.isFolder()) { // 如果是目录
			loadDir(fileD, list, isLoad);
		} else { // 如果是文件
			loadJar(fileD, list, isLoad);
		}
	}

	private void loadDir(FileObject fileD, List<BusinessUnitInfo> list,
			boolean isLoad) {
		logger.logMessage(LogLevel.DEBUG, "正在从目录[{0}]查找业务单元...",
				fileD.getAbsolutePath());
		List<FileObject> files = fileD.getChildren();
		for (FileObject f : files) {
			try {
				if (f.isFolder()) { // 如果是目录
					loadDir(f, list, isLoad);
				} else { // 如果是文件
					load(f, list, isLoad);
				}
			} catch (Exception e) {
				logger.errorMessage("读取文件[{0}]时出错", e, fileD.getAbsolutePath());
			}
		}
		logger.logMessage(LogLevel.DEBUG, "从目录[{0}]查找业务单元结束。",
				fileD.getAbsolutePath());

	}

	private void loadJar(FileObject f, List<BusinessUnitInfo> list,
			boolean isLoad) {
		logger.logMessage(LogLevel.DEBUG, "正在从文件[{0}]查找业务单元...",
				f.getAbsolutePath());
		try {
			load(f, list, isLoad);
		} catch (Exception e) {
			logger.errorMessage("读取文件{0}时出错", e, f.getAbsolutePath());
		}
		logger.logMessage(LogLevel.DEBUG, "从目录[{0}]查找业务单元结束。",
				f.getAbsolutePath());
	}

	private void load(FileObject f, List<BusinessUnitInfo> list, boolean isLoad)
			throws MalformedURLException {
		if (fileIsLoaded(f))
			return;
		if (!existBuFile(f))
			return;

		FileObject bUnit = f.getChild(BusinessUnitManager.BUSINESS_UNIT_FILE);
		XStream stream = XStreamFactory
				.getXStream(BusinessUnitManager.XSTEAM_PACKAGE_NAME);
		logger.logMessage(LogLevel.INFO, "在[{0}]找到业务单元文件", f.getAbsolutePath());
		BusinessUnitInfo businessUnitInfo = (BusinessUnitInfo) stream
				.fromXML(bUnit.getInputStream());
		list.add(businessUnitInfo);
		if (isLoad) {
			// 处理bu内容
			BusinessUnitManagerFactory.getManager().addBusinessUnit(f);
			// 20130115
			// loadBuInfo(f, newFileObject, businessUnitInfo);
		}

	}

	/**
	 * 读取bu的相关信息
	 * 
	 * @param f
	 * @param buJarFile
	 * @param info
	 * @throws MalformedURLException 
	 */
	public void loadBuInfo(FileObject f, BusinessUnitInfo info) throws MalformedURLException {

		if (!fileDependJar.containsKey(f)) {
			fileDependJar.put(f, new ArrayList<URL>());
		}
		fileUrl.put(f, f.getURL());
		for (BusinessUnitJarDependency jarDep : info
				.getBusinessUnitJarDependencies()) {
			String name = jarDep.getName();
			if (name == null || "".equals(name.trim())) {
				continue;
			}
			String urlString = f.getURL().toString()
					.replace(f.getFileName(), name);
			URL jarUrl = new URL(urlString);
			fileDependJar.get(f).add(jarUrl);
			logger.logMessage(LogLevel.INFO, "添加jar包{0}全路径{1}", name, jarUrl);
		}
		// 处理外部包
		if (!fileExtendJar.containsKey(f)) {
			fileExtendJar.put(f, new ArrayList<URL>());
		}
		for (BusinessUnitJarExtend jarExtend : info.getBusinessUnitJarExtends()) {
			String name = jarExtend.getName();
			if (name == null || "".equals(name.trim())) {
				continue;
			}
			String urlString = f.getURL().toString()
					.replace(f.getFileName(), name);
			URL jarUrl = new URL(urlString);
			fileExtendJar.get(f).add(jarUrl);
			logger.logMessage(LogLevel.INFO, "添加jar包{0}全路径{1}", name, jarUrl);
		}
		generateLoader(f);
	}

	/**
	 * 判断文件是否包含BU配置文件的jar包，即是否是bu Jar包
	 * 
	 * @param fileObject
	 * @return
	 */
	private boolean existBuFile(FileObject fileObject) {

		if (!fileObject.getFileName().endsWith(".jar")) {
			return false;
		}
		boolean exist = false;
		FileObject bUnit = fileObject
				.getChild(BusinessUnitManager.BUSINESS_UNIT_FILE);
		if (bUnit != null) {
			exist = true;
		}
		return exist;
	}

	private void generateLoader(FileObject f) {
		generatePublicLoader(f);
		generateBuLoaders(f);
	}

	/**
	 * 生成公共ClassLoader
	 */
	private void generatePublicLoader(FileObject f) {
		// 注册loader,add方法会自己过滤重复添加
		LoaderManagerFactory.getManager().addLoader(publicLoader);
		logger.logMessage(LogLevel.INFO, "开始生成所有外部包的Loader");
		List<URL> extendJars = new ArrayList<URL>();
		List<URL> listUrl = fileExtendJar.get(f);
		if (listUrl == null) {
			return;
		}
		for (URL url : listUrl) {
			if (!extendJars.contains(url)) {
				extendJars.add(url);
			}
		}
		URL urlArray[] = listToArray(extendJars, extendJars.size());
		publicLoader.addURL(urlArray);
		logger.logMessage(LogLevel.INFO, "所有外部包的Loader生成完毕");
		resolveResource(publicLoader, urlArray);
	}

	/**
	 * 为单个bu生成ClassLoader
	 */
	private void generateBuLoaders(FileObject file) {
		buLoaders.clear();
		List<URL> dependJar = fileDependJar.get(file);
		URL urlArray[] = null;
		int index = 0;
		if (dependJar == null) {
			urlArray = new URL[1];
		} else {
			urlArray = listToArray(dependJar, dependJar.size() + 1);
			index = dependJar.size();
		}
		urlArray[index] = fileUrl.get(file);
		URLClassLoader loader = new URLClassLoader(urlArray, publicLoader);
		// 注册loader
		LoaderManagerFactory.getManager().addLoader(loader);
		buLoaders.put(file, loader);
		resolveResource(loader, urlArray);
	}

	/**
	 * 添加BU时 读取指定jar包中的资源
	 * 
	 * @param loader
	 *            jar包对应的ClassLoader
	 * @param urls
	 *            jar包的url数组
	 */
	private void resolveResource(ClassLoader loader, URL[] urls) {

		resoler.resolve(urls, loader);
		PluginResourceFactory.getManager().register(loader, urls);
	}

	/**
	 * 移除BU时 移除指定jar包中的资源
	 * 
	 * @param loader
	 *            jar包对应的ClassLoader
	 * @param urls
	 *            jar包的url数组
	 */
	private void deResolveResource(ClassLoader loader, URL urls[]) {
		resoler.deResolve(urls, loader);
		PluginResourceFactory.getManager().remove(loader, urls);
	}

	/**
	 * 判断文件是否已被加载 若文件被加载返回true 否则返回false
	 * 
	 * @param f
	 * @return
	 */
	private boolean fileIsLoaded(FileObject f) {
		String fUrl = f.getURL().toString();
		for (FileObject o : fileDependJar.keySet()) {
			String oUrl = o.getURL().toString();
			if (fUrl.equals(oUrl)) {
				logger.logMessage(LogLevel.INFO, "添加jar包{0}已被加载过,不再加载", fUrl);
				return true;
			}

		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.tinygroup.bu.loader.BusinessUnitLoader#remove(org.apache
	 * .commons.vfs2.FileObject)
	 */
	public void remove(FileObject f) {
		logger.logMessage(LogLevel.INFO, "开始移除包{0}及相关资源", f.getAbsolutePath());
		removeDependJars(f);
		removeExtendJarResource(f);
		logger.logMessage(LogLevel.INFO, "移除包{0}及相关资源完成", f.getAbsolutePath());
	}

	/**
	 * 移除BU的外部包及相关资源
	 * 
	 * @param f
	 */
	private void removeExtendJarResource(FileObject f) {
		// 业务单元的公共包
		List<URL> removeExtendJars = getRemoveExtendJars(f);
		if (removeExtendJars == null) {
			return;
		}
		URL urlArray[] = listToArray(removeExtendJars, removeExtendJars.size());
		deResolveResource(publicLoader, urlArray);
	}

	/**
	 * 获取BU需要移除的外部包列表
	 * 
	 * @param f
	 * @return
	 */
	private List<URL> getRemoveExtendJars(FileObject f) {
		List<URL> extendJars = fileExtendJar.remove(f);
		if (extendJars == null)
			return null;
		// 剩余所有业务单元所依赖的包
		List<URL> totalExtendJars = new ArrayList<URL>();
		for (List<URL> list : fileExtendJar.values()) {
			totalExtendJars.addAll(list);
		}
		for (URL url : extendJars) {
			if (totalExtendJars.contains(url)) {
				extendJars.remove(url);
			}
		}
		return extendJars;
	}

	/**
	 * 移除BU的依赖包及资源
	 * 
	 * @param f
	 */
	private void removeDependJars(FileObject f) {
		List<URL> list = fileDependJar.remove(f);
		URL fUrl = f.getURL();
		URL urlArray[] = null;
		if (list == null) {
			urlArray = new URL[1];
			urlArray[0] = fUrl;
		} else {
			urlArray = listToArray(list, list.size() + 1);
			urlArray[list.size()] = fUrl;
		}
		ClassLoader loader = buLoaders.remove(f);
		deResolveResource(loader, urlArray);
	}

	/**
	 * 转换URLList为数组
	 * 
	 * @param urlList
	 * @param size
	 * @return
	 */
	private URL[] listToArray(List<URL> urlList, int size) {
		URL url[] = new URL[size];
		int i = 0;
		for (URL u1 : urlList) {
			url[i] = u1;
			i++;
		}
		return url;
	}

}
